@@ -40,9 +40,6 @@
 #ifdef HAVE_NETINET_IN_H
 # include <netinet/in.h>
 #endif
-#ifdef HAVE_RULI_H
-# include <ruli.h>
-#endif
 #ifdef HAVE_ERRNO_H
 # include <errno.h>
 #endif
@@ -130,8 +127,8 @@ unsigned long getaddress(char *host) {
 	*/
 	pent = gethostbyname(host);
 	if (!pent) {
-		printf("'%s' is unresolveable\n", host);
-		exit_code(2, __PRETTY_FUNCTION__, "hostname is not resolveable");
+		printf("'%s' is unresolvable\n", host);
+		exit_code(2, __PRETTY_FUNCTION__, "hostname is not resolvable");
 	}
 	addr = *(uint32_t *) (pent->h_addr);
 	return addr;
@@ -370,78 +367,11 @@ static inline unsigned long srv_ares(char *host, int *port, char *srv) {
 }
 #endif // HAVE_CARES_H
 
-#ifdef HAVE_RULI_H
-static inline unsigned long srv_ruli(char *host, int *port, char *srv) {
-	int srv_code;
-	int ruli_opts = RULI_RES_OPT_SEARCH | RULI_RES_OPT_SRV_NOINET6 | RULI_RES_OPT_SRV_NOSORT6 | RULI_RES_OPT_SRV_NOFALL;
-#ifdef RULI_RES_OPT_SRV_CNAME
-	ruli_opts |= RULI_RES_OPT_SRV_CNAME;
-#endif
-
-	ruli_sync_t *sync_query = ruli_sync_query(srv, host, *port, ruli_opts);
-
-	/* sync query failure? */
-	if (!sync_query) {
-		printf("DNS SRV lookup failed for: %s\n", host);
-		exit_code(2, __PRETTY_FUNCTION__, "DNS SRV lookup failed");
-	}
-
-	srv_code = ruli_sync_srv_code(sync_query);
-	/* timeout? */
-	if (srv_code == RULI_SRV_CODE_ALARM) {
-		printf("Timeout during DNS SRV lookup for: %s\n", host);
-		ruli_sync_delete(sync_query);
-		exit_code(2, __PRETTY_FUNCTION__, "timeout during DNS SRV lookup");
-	}
-	/* service provided? */
-	else if (srv_code == RULI_SRV_CODE_UNAVAILABLE) {
-		printf("SRV service not provided for: %s\n", host);
-		ruli_sync_delete(sync_query);
-		exit_code(2, __PRETTY_FUNCTION__, "missing service in DNS SRV reply");
-	}
-	else if (srv_code) {
-		int rcode = ruli_sync_rcode(sync_query);
-		if (verbose > 1)
-			printf("SRV query failed for: %s, srv_code=%d, rcode=%d\n", host, srv_code, rcode);
-		ruli_sync_delete(sync_query);
-		return 0;
-	}
-
-	ruli_list_t *srv_list = ruli_sync_srv_list(sync_query);
-
-	int srv_list_size = ruli_list_size(srv_list);
-
-	if (srv_list_size < 1) {
-		if (verbose > 1)
-			printf("No SRV record: %s.%s\n", srv, host);
-		return 0;
-	}
-
-	ruli_srv_entry_t *entry = (ruli_srv_entry_t *) ruli_list_get(srv_list, 0);
-	ruli_list_t *addr_list = &entry->addr_list;
-	int addr_list_size = ruli_list_size(addr_list);
-
-	if (addr_list_size < 1) {
-		printf("missing addresses in SRV lookup for: %s\n", host);
-		ruli_sync_delete(sync_query);
-		exit_code(2, __PRETTY_FUNCTION__, "missing address in DNS SRV reply");
-	}
-
-	*port = entry->port;
-	ruli_addr_t *addr = (ruli_addr_t *) ruli_list_get(addr_list, 0);
-	return addr->addr.ipv4.s_addr;
-}
-#endif // HAVE_RULI_H
-
 unsigned long getsrvaddress(char *host, int *port, char *srv) {
-#ifdef HAVE_RULI_H
-	return srv_ruli(host, port, srv);
-#else
-# ifdef HAVE_CARES_H // HAVE_RULI_H
+#ifdef HAVE_CARES_H
 	return srv_ares(host, port, srv);
-# else // HAVE_CARES_H
+#else // HAVE_CARES_H
 	return 0;
-# endif
 #endif
 }
 
@@ -510,7 +440,7 @@ unsigned long getsrvadr(char *host, int *port, unsigned int *transport) {
 /* because the full qualified domain name is needed by many other
    functions it will be determined by this function.
 */
-void get_fqdn() {
+void get_fqdn(char *buf, int numeric, char *hostname) {
 	char hname[100], dname[100], hlp[18];
 	size_t namelen=100;
 	struct hostent* he;
@@ -521,7 +451,7 @@ void get_fqdn() {
 	memset(&hlp, 0, sizeof(hlp));
 
 	if (hostname) {
-		strncpy(fqdn, hostname, FQDN_SIZE-1);
+		strncpy(buf, hostname, FQDN_SIZE-1);
 		strncpy(hname, hostname, sizeof(hname)-1);
 	}
 	else {
@@ -542,27 +472,27 @@ void get_fqdn() {
 				exit_code(2, __PRETTY_FUNCTION__, "failed to get domainname");
 			}
 			if (strcmp(&dname[0],"(none)")!=0)
-				snprintf(fqdn, FQDN_SIZE, "%s.%s", hname, dname);
+				snprintf(buf, FQDN_SIZE, "%s.%s", hname, dname);
 		}
 		else {
-			strncpy(fqdn, hname, FQDN_SIZE-1);
+			strncpy(buf, hname, FQDN_SIZE-1);
 		}
 #endif
 	}
 
-	if (!(numeric == 1 && is_ip(fqdn))) {
+	if (!(numeric == 1 && is_ip(buf))) {
 		he=gethostbyname(hname);
 		if (he) {
 			if (numeric == 1) {
 				snprintf(hlp, sizeof(hlp), "%s", inet_ntoa(*(struct in_addr *) he->h_addr_list[0]));
-				strncpy(fqdn, hlp, FQDN_SIZE-1);
+				strncpy(buf, hlp, FQDN_SIZE-1);
 			}
 			else {
 				if ((strchr(he->h_name, '.'))!=NULL && (strchr(hname, '.'))==NULL) {
-					strncpy(fqdn, he->h_name, FQDN_SIZE-1);
+					strncpy(buf, he->h_name, FQDN_SIZE-1);
 				}
 				else {
-					strncpy(fqdn, hname, FQDN_SIZE-1);
+					strncpy(buf, hname, FQDN_SIZE-1);
 				}
 			}
 		}
@@ -571,19 +501,19 @@ void get_fqdn() {
 			exit_code(2, __PRETTY_FUNCTION__, "failed to resolve local hostname");
 		}
 	}
-	if ((strchr(fqdn, '.'))==NULL) {
+	if ((strchr(buf, '.'))==NULL) {
 		if (hostname) {
-			fprintf(stderr, "warning: %s is not resolvable... continouing anyway\n", fqdn);
-			strncpy(fqdn, hostname, FQDN_SIZE-1);
+			fprintf(stderr, "warning: %s is not resolvable... continuing anyway\n", buf);
+			strncpy(buf, hostname, FQDN_SIZE-1);
 		}
 		else {
-			fprintf(stderr, "error: this FQDN or IP is not valid: %s\n", fqdn);
+			fprintf(stderr, "error: this FQDN or IP is not valid: %s\n", buf);
 			exit_code(2, __PRETTY_FUNCTION__, "invalid IP or FQDN");
 		}
 	}
 
 	if (verbose > 2)
-		printf("fqdnhostname: %s\n", fqdn);
+		printf("fqdnhostname: %s\n", buf);
 }
 
 /* this function searches for search in mess and replaces it with
@@ -619,7 +549,7 @@ void replace_strings(char *mes, char *strings) {
 	dbg("replace_strings entered\nstrings: '%s'\n", strings);
 	if ((isalnum(*strings) != 0) && 
 		(isalnum(*(strings + strlen(strings) - 1)) != 0)) {
-		replace_string(request, "$replace$", replace_str);
+		replace_string(mes, "$replace$", strings);
 	}
 	else {
 		sep = *strings;
@@ -847,7 +777,7 @@ void *str_alloc(size_t size) {
 	ptr = malloc(size);
 #endif
 	if (ptr == NULL) {
-		fprintf(stderr, "error: memory allocation failed\n");
+		fprintf(stderr, "error: memory allocation for %lu bytes failed\n", size);
 		exit_code(255, __PRETTY_FUNCTION__, "memory allocation failure");
 	}
 #ifndef HAVE_CALLOC
