@@ -100,10 +100,24 @@
 #include "helper.h"
 #include "header_f.h"
 
+char *transport_str;
+char target_dot[INET_ADDRSTRLEN], source_dot[INET_ADDRSTRLEN];
+
 #ifdef RAW_SUPPORT
 int rawsock;
 #endif
 
+#ifdef USE_GNUTLS
+gnutls_session_t tls_session;
+//gnutls_anon_client_credentials_t anoncred;
+gnutls_certificate_credentials_t xcred;
+#else
+# ifdef USE_OPENSSL
+SSL_CTX* ctx;
+SSL* ssl;
+# endif
+#endif
+
 #ifdef WITH_TLS_TRANSP
 # ifdef USE_GNUTLS
 void check_alert(gnutls_session_t session, int ret) {
@@ -170,7 +184,7 @@ static void verify_cert2(gnutls_x509_crt_t crt, gnutls_x509_crt_t issuer,
 
 	/* Now check the expiration dates */
 	if (gnutls_x509_crt_get_activation_time(crt) > now) {
-		printf("Certificate not yet activated!\n");
+		printf("Certificate is not yet valid!\n");
 	}
 	if (gnutls_x509_crt_get_expiration_time(crt) < now) {
 		printf("Certificate expired!\n");
@@ -204,7 +218,7 @@ static void verify_last_cert(gnutls_x509_crt_t crt, gnutls_x509_crt_t *ca_list,
 	gnutls_x509_crt_verify(crt, ca_list, ca_list_size, 
 			GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT, &output);
 	if (output & GNUTLS_CERT_INVALID) {
-		printf("Certificate not truested!\n");
+		printf("Certificate not trusted!\n");
 		if (output & GNUTLS_CERT_SIGNER_NOT_CA) {
 			printf(": Issuer is not a CA\n");
 		}
@@ -215,7 +229,7 @@ static void verify_last_cert(gnutls_x509_crt_t crt, gnutls_x509_crt_t *ca_list,
 
 	/* Now check the expiration dates */
 	if (gnutls_x509_crt_get_activation_time(crt) > now) {
-		printf("Certificate now yet activated!\n");
+		printf("Certificate is not yet valid!\n");
 	}
 	if (gnutls_x509_crt_get_expiration_time(crt) < now) {
 		printf("Certificate expired!\n");
@@ -236,7 +250,7 @@ void verify_certificate_chain(gnutls_session_t session, const char *hostname,
 
 	cert = malloc(sizeof(*cert) * cert_chain_length);
 	if (!cert) {
-		printf("gnutla: failed to allocate memory for cert chain verification'n");
+		printf("gnutls: failed to allocate memory for cert chain verification'n");
 		return;
 	}
 
@@ -274,7 +288,8 @@ void verify_certificate_chain(gnutls_session_t session, const char *hostname,
 	return;
 }
 
-int verify_certificate_simple(gnutls_session_t session, const char *hostname) {
+int verify_certificate_simple(gnutls_session_t session, const char *hostname,
+    int ignore_ca_fail) {
 	unsigned int status, cert_list_size;
 	const gnutls_datum_t *cert_list;
 	int ret;
@@ -303,7 +318,7 @@ int verify_certificate_simple(gnutls_session_t session, const char *hostname) {
 		}
 	}
 	if (status & GNUTLS_CERT_REVOKED) {
-		printf("The certificate has beend revoked.\n");
+		printf("The certificate has been revoked.\n");
 		ret = -16;
 	}
 	if (ret != 0 && ignore_ca_fail == 0) {
@@ -322,7 +337,7 @@ int verify_certificate_simple(gnutls_session_t session, const char *hostname) {
 
 	cert_list = gnutls_certificate_get_peers(session, &cert_list_size);
 	if (cert_list == NULL) {
-		printf("gnutls did not found a server certificate.\n");
+		printf("gnutls did not find a server certificate.\n");
 		return -128;
 	}
 
@@ -338,7 +353,7 @@ int verify_certificate_simple(gnutls_session_t session, const char *hostname) {
 		return -512;
 	}
 	if (gnutls_x509_crt_get_activation_time(cert) > time(0)) {
-		printf("The server certificate is not yet activated.\n");
+		printf("The server certificate is not yet valid.\n");
 		return -1024;
 	}
 	if (!gnutls_x509_crt_check_hostname(cert, hostname)) {
@@ -383,7 +398,7 @@ void print_x509_certificate_info(gnutls_session_t session) {
 
 	// check if we got a X.509 cert
 	if (gnutls_certificate_type_get(session) != GNUTLS_CRT_X509) {
-		printf("TLS session did not received a X.509 certificate\n");
+		printf("TLS session did not receive a X.509 certificate\n");
 		return;
 	}
 
@@ -391,7 +406,7 @@ void print_x509_certificate_info(gnutls_session_t session) {
 	printf("Peer provided %u certificate(s)\n", cert_list_size);
 
 	if (cert_list_size > 0) {
-		// print only informations about the first cert
+		// print only information about the first cert
 		gnutls_x509_crt_init(&cert);
 		gnutls_x509_crt_import(cert, &cert_list[0], GNUTLS_X509_FMT_DER);
 		printf("Certificate info:\n");
@@ -402,7 +417,7 @@ void print_x509_certificate_info(gnutls_session_t session) {
 		// print the serial number of the certificate
 		size = sizeof(serial);
 		gnutls_x509_crt_get_serial(cert, serial, &size);
-		printf("\tCertificate serail number: %s\n", bin2hex(serial, size));
+		printf("\tCertificate serial number: %s\n", bin2hex(serial, size));
 		// extract public key algorithm
 		algo = gnutls_x509_crt_get_pk_algorithm(cert, &bits);
 		printf("\tCertificate public key algorithm: %s\n", gnutls_pk_algorithm_get_name(algo));
@@ -449,7 +464,7 @@ void gnutls_session_info(gnutls_session_t session) {
 	// print the key exchange algorithm name
 	kx = gnutls_kx_get(session);
 	tmp = gnutls_kx_get_name(kx);
-	printf("Key Echange: %s\n", tmp);
+	printf("Key Exchange: %s\n", tmp);
 
 	// check the authentication type
 	cred = gnutls_auth_get_type(session);
@@ -467,7 +482,7 @@ void gnutls_session_info(gnutls_session_t session) {
 		case GNUTLS_CRD_CERTIFICATE:
 			// check if we have been using ephemeral DH
 			if (kx == GNUTLS_KX_DHE_RSA || kx == GNUTLS_KX_DHE_DSS) {
-				printf("Emphemeral DH using prime of %d bits\n",
+				printf("Ephemeral DH using prime of %d bits\n",
 					gnutls_dh_get_prime_bits(session));
 			}
 			// print certificate informations if available
@@ -555,25 +570,74 @@ void tls_dump_cert_info(char* s, X509* cert) {
 # endif /* USE_GNUTLS */
 #endif /* WITH_TLS_TRANSP */
 
-void create_sockets(struct sipsak_con_data *cd) {
+void init_network(struct sipsak_con_data *cd, char *local_ip
+#ifdef WITH_TLS_TRANSP
+    , char *ca_file
+#endif
+    ) {
 	socklen_t slen;
 
+  transport_str = NULL;
+  memset(&target_dot, 0, INET_ADDRSTRLEN);
+  memset(&source_dot, 0, INET_ADDRSTRLEN);
 #ifdef RAW_SUPPORT
 	rawsock = -1;
 #endif
 
+	switch (cd->transport) {
+#ifdef WITH_TLS_TRANSP
+		case SIP_TLS_TRANSPORT:
+			transport_str = TRANSPORT_TLS_STR;
+			break;
+#endif /* WITH_TLS_TRANSP */
+		case SIP_TCP_TRANSPORT:
+			transport_str = TRANSPORT_TCP_STR;
+			break;
+		case SIP_UDP_TRANSPORT:
+			transport_str = TRANSPORT_UDP_STR;
+			break;
+		default:
+			fprintf(stderr, "unknown transport: %u\n", cd->transport);
+			exit_code(2, __PRETTY_FUNCTION__, "unknown transport");
+	}
+
+#ifdef WITH_TLS_TRANSP
+	if (cd->transport == SIP_TLS_TRANSPORT) {
+# ifdef USE_GNUTLS
+    tls_session = NULL;
+    xcred = NULL;
+
+		gnutls_global_init();
+		//gnutls_anon_allocate_client_credentials(&anoncred);
+		gnutls_certificate_allocate_credentials(&xcred);
+		if (ca_file != NULL) {
+			// set the trusted CA file
+			gnutls_certificate_set_x509_trust_file(xcred, ca_file, GNUTLS_X509_FMT_PEM);
+		}
+# else
+#  ifdef USE_OPENSSL
+    ctx = NULL;
+    ssl = NULL;
+
+		SSL_library_init();
+		SSL_load_error_strings();
+#  endif
+# endif
+	}
+#endif /* WITH_TLS_TRANSP */
+
 	memset(&(cd->adr), 0, sizeof(struct sockaddr_in));
 	cd->adr.sin_family = AF_INET;
 	if(local_ip) {	
 		cd->adr.sin_addr.s_addr = inet_addr(local_ip);
 	} else {
-		cd->adr.sin_addr.s_addr = htonl( INADDR_ANY);
+		cd->adr.sin_addr.s_addr = htonl(INADDR_ANY);
 	}
-	cd->adr.sin_port = htons((short)lport);
+	cd->adr.sin_port = htons((short)cd->lport);
 
-	if (transport == SIP_UDP_TRANSPORT) {
+	if (cd->transport == SIP_UDP_TRANSPORT) {
 		/* create the un-connected socket */
-		if (!symmetric) {
+		if (!cd->symmetric) {
 			cd->usock = (int)socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
 			if (cd->usock==-1) {
 				perror("unconnected UDP socket creation failed");
@@ -585,7 +649,6 @@ void create_sockets(struct sipsak_con_data *cd) {
 			}
 		}
 
-
 #ifdef RAW_SUPPORT
 		/* try to create the raw socket */
 		rawsock = (int)socket(PF_INET, SOCK_RAW, IPPROTO_ICMP);
@@ -601,7 +664,7 @@ void create_sockets(struct sipsak_con_data *cd) {
 				exit_code(2, __PRETTY_FUNCTION__, "failed to create connected UDP socket");
 			}
 
-			if (!symmetric)
+			if (!cd->symmetric)
 				cd->adr.sin_port = htons((short)0);
 			if (bind(cd->csock, (struct sockaddr *) &(cd->adr), sizeof(struct sockaddr_in) )==-1) {
 				perror("connected UDP socket binding failed");
@@ -609,7 +672,7 @@ void create_sockets(struct sipsak_con_data *cd) {
 			}
 #ifdef RAW_SUPPORT
 		}
-		else if (symmetric) {
+		else if (cd->symmetric) {
 			cd->csock = (int)socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
 			if (cd->csock==-1) {
 				perror("connected UDP socket creation failed");
@@ -633,7 +696,7 @@ void create_sockets(struct sipsak_con_data *cd) {
 			exit_code(2, __PRETTY_FUNCTION__, "failed to bind TCP socket");
 		}
 #ifdef WITH_TLS_TRANSP
-		if (transport == SIP_TLS_TRANSPORT) {
+		if (cd->transport == SIP_TLS_TRANSPORT) {
 #ifdef USE_GNUTLS
 			// initialixe the TLS session
 			gnutls_init(&tls_session, GNUTLS_CLIENT);
@@ -667,17 +730,32 @@ void create_sockets(struct sipsak_con_data *cd) {
 	}
 
 	/* for the via line we need our listening port number */
-	if (lport==0){
+	if (cd->lport==0){
 		memset(&(cd->adr), 0, sizeof(struct sockaddr_in));
 		slen=sizeof(struct sockaddr_in);
-		if (symmetric || transport != SIP_UDP_TRANSPORT)
+		if (cd->symmetric || cd->transport != SIP_UDP_TRANSPORT)
 			getsockname(cd->csock, (struct sockaddr *) &(cd->adr), &slen);
 		else
 			getsockname(cd->usock, (struct sockaddr *) &(cd->adr), &slen);
-		lport=ntohs(cd->adr.sin_port);
+		cd->lport=ntohs(cd->adr.sin_port);
 	}
 }
 
+void shutdown_network() {
+# ifdef USE_GNUTLS
+  if (tls_session) {
+    gnutls_deinit(tls_session);
+  }
+  if (xcred) {
+    gnutls_certificate_free_credentials(xcred);
+  }
+  gnutls_global_deinit();
+# else /* USE_GNUTLS */
+#  ifdef USE_OPENSSL
+#  endif /* USE_OPENSSL */
+# endif /* USE_GNUTLS */
+}
+
 void send_message(char* mes, struct sipsak_con_data *cd,
 			struct sipsak_counter *sc, struct sipsak_sr_time *srt) {
 	struct timezone tz;
@@ -695,7 +773,7 @@ void send_message(char* mes, struct sipsak_con_data *cd,
 		else {
 			dbg("\nusing connected socket for sending\n");
 #ifdef WITH_TLS_TRANSP
-			if (transport == SIP_TLS_TRANSPORT) {
+			if (cd->transport == SIP_TLS_TRANSPORT) {
 # ifdef USE_GNUTLS
 				ret = gnutls_record_send(tls_session, mes, strlen(mes));
 # else /* USE_GNUTLS */
@@ -719,7 +797,7 @@ void send_message(char* mes, struct sipsak_con_data *cd,
 		}
 #ifdef HAVE_INET_NTOP
 		if (verbose > 2) {
-			printf("\nsend to: %s:%s:%i\n", transport_str, target_dot, rport);
+			printf("\nsend to: %s:%s:%i\n", transport_str, target_dot, cd->rport);
     }
 #endif
 		sc->send_counter++;
@@ -729,7 +807,8 @@ void send_message(char* mes, struct sipsak_con_data *cd,
 	}
 }
 
-void check_socket_error(int socket, int size) {
+void check_socket_error(int socket, char *buffer, int size,
+    enum sipsak_modes mode, char *request) {
 	struct pollfd sockerr;
 	int ret = 0;
 
@@ -739,11 +818,11 @@ void check_socket_error(int socket, int size) {
 	ret = poll(&sockerr, 1, 10);
 	if (ret==1) {
 		if (sockerr.revents & POLLERR) {
-			recvfrom(socket, recv, size, 0, NULL, 0);
+			recvfrom(socket, buffer, size, 0, NULL, 0);
 			if (verbose)
 				printf("\n");
 			perror("send failure");
-			if (randtrash == 1) {
+			if (mode == SM_RANDTRASH) {
 				printf ("last message before send failure:\n%s\n", request);
 				log_message(request);
 			}
@@ -754,8 +833,9 @@ void check_socket_error(int socket, int size) {
 
 int check_for_message(char *recv, int size, struct sipsak_con_data *cd,
 			struct sipsak_sr_time *srt, struct sipsak_counter *count,
-			struct sipsak_delay *sd) {
-	fd_set	fd;
+			struct sipsak_delay *sd, enum sipsak_modes mode, int cseq_counter,
+      char *request, char *response, int inv_trans) {
+	fd_set fd;
 	struct timezone tz;
 	struct timeval tv;
 	double senddiff;
@@ -787,33 +867,35 @@ int check_for_message(char *recv, int size, struct sipsak_con_data *cd,
 	if (count->send_counter==1) {
 		memcpy(&(srt->firstsendt), &(srt->sendtime), sizeof(struct timeval));
 	}
-	if (sd->retryAfter == timer_t1) {
+	if (sd->retryAfter == srt->timer_t1) {
 		memcpy(&(srt->starttime), &(srt->sendtime), sizeof(struct timeval));
 	}
 	if (ret == 0)
 	{
 		/* lets see if we at least received an icmp error */
 		if (cd->csock == -1) 
-			check_socket_error(cd->usock, size);
+			check_socket_error(cd->usock, recv, size, mode, request);
 		else
-			check_socket_error(cd->csock, size);
+			check_socket_error(cd->csock, recv, size, mode, request);
 		/* printout that we did not received anything */
 		if (verbose > 0) {
-			if (trace == 1) {
-				printf("%i: timeout after %i ms\n", namebeg, sd->retryAfter);
+			if (mode == SM_TRACE) {
+				printf("%i: timeout after %i ms\n", count->namebeg, sd->retryAfter);
 			}
-			else if (usrloc == 1||invite == 1||message == 1) {
+			else if (mode == SM_USRLOC ||
+               mode == SM_INVITE ||
+               mode == SM_MESSAGE) {
 				printf("timeout after %i ms\n", sd->retryAfter);
 			}
 			else {
 				printf("** timeout after %i ms**\n", sd->retryAfter);
 			}
 		}
-		if (randtrash == 1) {
+		if (mode == SM_RANDTRASH) {
 			printf("did not get a response on this request:\n%s\n", request);
-			if (cseq_counter < nameend) {
+			if (cseq_counter < count->nameend) {
 				if (count->randretrys == 2) {
-					printf("sended the following message three "
+					printf("sent the following message three "
 							"times without getting a response:\n%s\n"
 							"give up further retransmissions...\n", request);
 					log_message(request);
@@ -827,21 +909,21 @@ int check_for_message(char *recv, int size, struct sipsak_con_data *cd,
 			}
 		}
 		senddiff = deltaT(&(srt->starttime), &(srt->recvtime));
-		if (senddiff > (float)timer_final) {
-			if (timing == 0) {
+		if (senddiff > (double)srt->timer_final) {
+			if (srt->timing == 0) {
 				if (verbose>0)
 					printf("*** giving up, no final response after %.3f ms\n", senddiff);
 				log_message(request);
 				exit_code(3, __PRETTY_FUNCTION__, "timeout (no final response)");
 			}
 			else {
-				timing--;
+				srt->timing--;
 				count->run++;
 				sd->all_delay += senddiff;
 				sd->big_delay = senddiff;
 				new_transaction(request, response);
-				sd->retryAfter = timer_t1;
-				if (timing == 0) {
+				sd->retryAfter = srt->timer_t1;
+				if (srt->timing == 0) {
 					printf("%.3f/%.3f/%.3f ms\n", sd->small_delay, sd->all_delay / count->run, sd->big_delay);
 					log_message(request);
 					exit_code(3, __PRETTY_FUNCTION__, "timeout (no final response)");
@@ -850,11 +932,11 @@ int check_for_message(char *recv, int size, struct sipsak_con_data *cd,
 		}
 		else {
 			/* set retry time according to RFC3261 */
-			if ((inv_trans) || (sd->retryAfter *2 < timer_t2)) {
+			if ((inv_trans) || (sd->retryAfter *2 < srt->timer_t2)) {
 				sd->retryAfter = sd->retryAfter * 2;
 			}
 			else {
-				sd->retryAfter = timer_t2;
+				sd->retryAfter = srt->timer_t2;
 			}
 		}
 		(count->retrans_s_c)++;
@@ -877,7 +959,7 @@ int check_for_message(char *recv, int size, struct sipsak_con_data *cd,
 			exit_code(2, __PRETTY_FUNCTION__, "failed to determine receiving socket");
 		}
 		/* no timeout, no error ... something has happened :-) */
-	 	if (trace == 0 && usrloc ==0 && invite == 0 && message == 0 && randtrash == 0 && (verbose > 1))
+		if ((mode == SM_FLOOD || mode == SM_UNDEFINED) && (verbose > 1))
 			printf ("\nmessage received");
 	}
 #ifdef RAW_SUPPORT
@@ -931,10 +1013,11 @@ int complete_mes(char *mes, int size) {
 	}
 }
 
-int recv_message(char *buf, int size, int inv_trans, 
+int recv_message(char *buf, int size, int inv_trans,
 			struct sipsak_delay *sd, struct sipsak_sr_time *srt,
 			struct sipsak_counter *count, struct sipsak_con_data *cd,
-			struct sipsak_regexp *reg) {
+			struct sipsak_regexp *reg, enum sipsak_modes mode, int cseq_counter,
+      char *request, char *response) {
 	int ret = 0;
 	int sock = 0;
 	double tmp_delay;
@@ -956,7 +1039,8 @@ int recv_message(char *buf, int size, int inv_trans,
 		buf = cd->buf_tmp;
 		size = size - cd->buf_tmp_size;
 	}
-	sock = check_for_message(buf, size, cd, srt, count, sd);
+	sock = check_for_message(buf, size, cd, srt, count, sd, mode, cseq_counter,
+      request, response, inv_trans);
 	if (sock <= 1) {
 		return -1;
 	}
@@ -965,9 +1049,9 @@ int recv_message(char *buf, int size, int inv_trans,
 #else
 	else {
 #endif
-		check_socket_error(sock, size);
+		check_socket_error(sock, buf, size, mode, request);
 #ifdef WITH_TLS_TRANSP
-		if (transport == SIP_TLS_TRANSPORT) {
+		if (cd->transport == SIP_TLS_TRANSPORT) {
 # ifdef USE_GNUTLS
 			ret = gnutls_record_recv(tls_session, buf, size);
 # else /* USE_GNUTLS */
@@ -1015,8 +1099,8 @@ int recv_message(char *buf, int size, int inv_trans,
 			udp_hdr = (struct udphdr *) ((char *)s_ip_hdr + s_ip_len);
 			srcport = ntohs(udp_hdr->uh_sport);
 			dstport = ntohs(udp_hdr->uh_dport);
-			dbg("\nlport: %i, rport: %i\n", lport, rport);
-			if ((srcport == lport) && (dstport == rport)) {
+			dbg("\nlport: %i, rport: %i\n", cd->lport, cd->rport);
+			if ((srcport == cd->lport) && (dstport == cd->rport)) {
 				printf(" (type: %u, code: %u)", icmp_hdr->icmp_type, icmp_hdr->icmp_code);
 #ifdef HAVE_INET_NTOP
 				if (inet_ntop(AF_INET, &faddr.sin_addr, &source_dot[0], INET_ADDRSTRLEN) != NULL)
@@ -1044,7 +1128,7 @@ int recv_message(char *buf, int size, int inv_trans,
 #endif // RAW_SUPPORT
 	if (ret > 0) {
 		*(buf+ ret) = '\0';
-		if (transport != SIP_UDP_TRANSPORT) {
+		if (cd->transport != SIP_UDP_TRANSPORT) {
 			if (verbose > 0)
 				printf("\nchecking message for completeness...\n");
 			if (complete_mes(buf, ret) == 1) {
@@ -1075,7 +1159,7 @@ int recv_message(char *buf, int size, int inv_trans,
 			srt->delaytime.tv_sec = 0;
 			srt->delaytime.tv_usec = 0;
 		}
-		if (timing > 0) {
+		if (srt->timing > 0) {
 			tmp_delay = deltaT(&(srt->sendtime), &(srt->recvtime));
 			if (tmp_delay > sd->big_delay)
 				sd->big_delay = tmp_delay;
@@ -1088,18 +1172,18 @@ int recv_message(char *buf, int size, int inv_trans,
 			printf("\nreceived from: %s:%s:%i\n", transport_str, 
 						source_dot, ntohs(peer_adr.sin_port));
 		}
-		else if (verbose > 1 && trace == 0 && usrloc == 0)
+		else if ((verbose > 1) && (mode != SM_TRACE && mode != SM_USRLOC))
 			printf(":\n");
 #else
-		if (trace == 0 && usrloc == 0)
+		if (mode != SM_TRACE && mode != SM_USRLOC)
 			printf(":\n");
 #endif // HAVE_INET_NTOP
 		if (!inv_trans && ret > 0 && (regexec(&(reg->proexp), buf, 0, 0, 0) != REG_NOERROR)) {
-			sd->retryAfter = timer_t1;
+			sd->retryAfter = srt->timer_t1;
 		}
 	}
 	else {
-		check_socket_error(sock, size);
+		check_socket_error(sock, buf, size, mode, request);
 		printf("\nnothing received, select returned error\n");
 		exit_code(2, __PRETTY_FUNCTION__, "nothing received, select returned error");
 	}
@@ -1108,7 +1192,12 @@ int recv_message(char *buf, int size, int inv_trans,
 
 /* clears the given sockaddr, fills it with the given data and if a
  * socket is given connects the socket to the new target */
-int set_target(struct sockaddr_in *adr, unsigned long target, int port, int socket, int connected) {
+int set_target(struct sockaddr_in *adr, unsigned long target, int port,
+    int socket, int connected, unsigned int transport, char *domainname
+#ifdef WITH_TLS_TRANSP
+    , int ignore_ca_fail
+#endif
+    ) {
 #ifdef WITH_TLS_TRANSP
 	int ret;
 # ifdef USE_OPENSSL
@@ -1152,7 +1241,7 @@ int set_target(struct sockaddr_in *adr, unsigned long target, int port, int sock
 			else if (verbose > 2) {
 				dbg(" TLS Handshake was completed!\n");
 				gnutls_session_info(tls_session);
-				if (verify_certificate_simple(tls_session, domainname) != 0) {
+				if (verify_certificate_simple(tls_session, domainname, ignore_ca_fail) != 0) {
 					if (ignore_ca_fail == 1) {
 						if (verbose) {
 							printf("WARN: Ignoring verification failures of the server certificate\n");
@@ -1180,7 +1269,7 @@ int set_target(struct sockaddr_in *adr, unsigned long target, int port, int sock
 				if (cert != 0) {
 					tls_dump_cert_info("TLS connect: server certificate", cert);
 					if (SSL_get_verify_result(ssl) != X509_V_OK) {
-						perror("TLS connect: server certifcate verification failed!!!\n");
+						perror("TLS connect: server certificate verification failed!!!\n");
 						exit_code(3, __PRETTY_FUNCTION__, "TLS server certificate verification falied");
 					}
 					X509_free(cert);
@@ -1194,7 +1283,7 @@ int set_target(struct sockaddr_in *adr, unsigned long target, int port, int sock
 				err = SSL_get_error(ssl, ret);
 				switch (err) {
 					case SSL_ERROR_ZERO_RETURN:
-						perror("TLS handshakre failed cleanly'n");
+						perror("TLS handshake failed cleanly'n");
 						break;
 					case SSL_ERROR_WANT_READ:
 						perror("Need to get more data to finish TLS connect\n");
